@@ -1,33 +1,29 @@
-/*
-** server.c - YP server routines.
-**
-** Copyright (c) 1996, 1997, 1998 Thorsten Kukuk
-** Copyright (c) 1993 Signum Support AB, Sweden
-**
-** This file is part of the NYS YP Server.
-**
-** The NYS YP Server is free software; you can redistribute it and/or
-** modify it under the terms of the GNU General Public License as
-** published by the Free Software Foundation; either version 2 of the
-** License, or (at your option) any later version.
-**
-** The NYS YP Server is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-** General Public License for more details.
-**
-** You should have received a copy of the GNU General Public
-** License along with the NYS YP Server; see the file COPYING.  If
-** not, write to the Free Software Foundation, Inc., 675 Mass Ave,
-** Cambridge, MA 02139, USA.
-**
-** Author: Peter Eriksson <pen@signum.se>
-**         Thorsten Kukuk <kukuk@uni-paderborn.de>
-*/
+/* server.c - YP server routines.
 
-#ifndef LINT
-static const char rcsid[] = "$Id: server.c,v 1.24 1998/10/10 08:16:53 kukuk Exp $";
-#endif
+   Copyright (c) 1996, 1997, 1998, 1999 Thorsten Kukuk, Germany
+   Copyright (c) 1993 Signum Support AB, Sweden
+
+   This file is part of the NYS YP Server.
+
+   The NYS YP Server is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License as
+   published by the Free Software Foundation; either version 2 of the
+   License, or (at your option) any later version.
+
+   The NYS YP Server is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   General Public License for more details.
+
+   You should have received a copy of the GNU General Public
+   License along with the NYS YP Server; see the file COPYING.  If
+   not, write to the Free Software Foundation, Inc., 675 Mass Ave,
+   Cambridge, MA 02139, USA.
+
+   Authors:
+   Peter Eriksson <pen@signum.se>
+   Thorsten Kukuk <kukuk@suse.de>
+*/
 
 #if defined(HAVE_CONFIG_H)
 #include "config.h"
@@ -69,6 +65,10 @@ int xfr_check_port = 1;
 #include <compat/strndup.c>
 #endif
 
+#ifndef HAVE_STPCPY
+#include <compat/stpcpy.c>
+#endif
+
 #if !defined(HAVE_GETHOSTNAME)
 #include <sys/utsname.h>
 
@@ -98,22 +98,22 @@ static int
 mangle_field (datum * val, int field)
 {
   int i, j, k, anz;
-  char *p;
-
   /* allocate a little bit more memory, it's safer, because the
      field could be empty */
-  if ((p = malloc (val->dsize + 3)) == NULL)
+  char *p = alloca (val->dsize + 3);
+
+  if (p == NULL)
     {
       yp_msg ("ERROR: could not allocate enough memory! [%s|%d]\n",
-              __FILE__, __LINE__);
+	      __FILE__, __LINE__);
       return -1;
     }
 
   anz = 0;
-  for (i = 0; i < val->dsize; i++)
+  for (i = 0; i < val->dsize; ++i)
     {
       if (val->dptr[i] == ':')
-        anz++;
+        ++anz;
       if (anz + 1 == field)
         {
           anz = i;
@@ -130,25 +130,25 @@ mangle_field (datum * val, int field)
               strcat (p, ":x");
               anz += 2;
             }
-          for (j = anz; j < val->dsize && val->dptr[j] != ':'; j++);
-          for (k = j; k < val->dsize; k++)
+          for (j = anz; j < val->dsize && val->dptr[j] != ':'; ++j);
+          for (k = j; k < val->dsize; ++k)
             {
               p[anz] = val->dptr[k];
-              anz++;
+              ++anz;
             }
           free (val->dptr);
-          val->dptr = p;
+          val->dptr = strdup (p);
           val->dsize = anz;
           return 0;
         }
     }
-  free (p);
   return 0;
 }
 
 
 void *
-ypproc_null_2_svc (void *dummy, struct svc_req *rqstp)
+ypproc_null_2_svc (void *dummy __attribute__ ((unused)),
+		   struct svc_req *rqstp)
 {
   static char *result;
   static char rval = 0;
@@ -170,7 +170,7 @@ ypproc_null_2_svc (void *dummy, struct svc_req *rqstp)
 }
 
 bool_t *
-ypproc_domain_2_svc (domainname * name, struct svc_req *rqstp)
+ypproc_domain_2_svc (domainname *name, struct svc_req *rqstp)
 {
   static bool_t result;
   struct sockaddr_in *rqhost;
@@ -243,10 +243,8 @@ ypproc_domain_nonack_2_svc (domainname * name,
   return &result;
 }
 
-/*
-** Get the DateTimeModified value for a certain map database
-*/
-static unsigned long
+/* Get the DateTimeModified value for a certain map database */
+static inline unsigned long
 get_dtm (const char *domain, const char *map)
 {
   struct stat sbuf;
@@ -260,9 +258,7 @@ get_dtm (const char *domain, const char *map)
     return (unsigned long) sbuf.st_mtime;
 }
 
-/*
-** YP function "MATCH" implementation
-*/
+/* YP function "MATCH" implementation */
 ypresp_val *
 ypproc_match_2_svc (ypreq_key *key, struct svc_req *rqstp)
 {
@@ -366,16 +362,38 @@ ypproc_match_2_svc (ypreq_key *key, struct svc_req *rqstp)
   ** DNS lookups are handled in a subprocess so that the server
   ** doesn't block while waiting for requests to complete.
   */
-  if (result.stat != YP_TRUE && strstr (key->map, "hosts") && dns_flag)
+  if (dns_flag && result.stat != YP_TRUE && strstr (key->map, "hosts"))
     {
       struct hostent *he;
-      if (children < MAX_CHILDREN && fork ())
+      if (children < MAX_CHILDREN)
 	{
-	  children++;
-	  return NULL;
+	  switch (fork ())
+	    {
+	    case 0:
+#ifdef DEBUG
+	      yp_msg ("ypserv has forked: pid=%i\n", getpid ());
+#endif
+	      ++forked;
+	      /* Close all databases ! */
+	      ypdb_close_all ();
+	      break;
+	    case -1:
+	      yp_msg ("Cannot fork: %s\n", strerror (errno));
+	      result.stat = YP_YPERR;
+	      return &result;
+	      break;
+	    default:
+	      ++children;
+	      return NULL;
+	      break;
+	    }
 	}
       else
-	forked++;
+	{
+	  yp_msg ("WARNING: ypserv has to many running children !\n");
+	  result.stat = YP_YPERR;
+	  return &result;
+	}
 
       key->key.keydat_val[key->key.keydat_len] = '\0';
 
@@ -401,20 +419,20 @@ ypproc_match_2_svc (ypreq_key *key, struct svc_req *rqstp)
 	  char *cp;
 
 	  if (debug_flag)
-	    yp_msg ("\t-> OK (%s/%s)\n",
-		    he->h_name,
+	    yp_msg ("\t-> OK (%s/%s)\n", he->h_name,
 		    inet_ntoa (*(struct in_addr *) he->h_addr));
 
-	  for (j = 0; he->h_addr_list[j]; j++)
+	  for (j = 0; he->h_addr_list[j]; ++j)
 	    ;
 
 	  len = 16;		/* Numerical IP address */
 	  len += strlen (he->h_name) + 1;
-	  for (i = 0; he->h_aliases[i]; i++)
+	  for (i = 0; he->h_aliases[i]; ++i)
 	    len += strlen (he->h_aliases[i]) + 1;
 
 	  len *= j;
-	  if ((cp = malloc (len + 1)) == NULL)
+	  cp = malloc (len + 1);
+	  if (cp == NULL)
 	    {
 	      yp_msg ("ERROR: could not allocate enough memory! [%s|%d]\n",
 		      __FILE__, __LINE__);
@@ -422,34 +440,36 @@ ypproc_match_2_svc (ypreq_key *key, struct svc_req *rqstp)
 	      return &result;
 	    }
 
-	  strcpy (cp, inet_ntoa (*(struct in_addr *) he->h_addr_list[0]));
-	  strcat (cp, "	");
-	  strcat (cp, he->h_name);
+	  result.val.valdat_val = cp;
 
-	  for (i = 0; he->h_aliases[i]; i++)
+	  cp = stpcpy (cp, inet_ntoa (*(struct in_addr *) he->h_addr_list[0]));
+	  *cp++ = '\t';
+	  cp = stpcpy (cp, he->h_name);
+
+	  for (i = 0; he->h_aliases[i]; ++i)
 	    {
-	      strcat (cp, " ");
-	      strcat (cp, he->h_aliases[i]);
+	      cp = stpcpy (cp, " ");
+	      cp = stpcpy (cp, he->h_aliases[i]);
 	    }
 
-	  for (j = 1; he->h_addr_list[j]; j++)
+	  for (j = 1; he->h_addr_list[j]; ++j)
 	    {
-	      strcat (cp, "\n");
-	      strcat (cp, inet_ntoa (*(struct in_addr *) he->h_addr_list[j]));
-	      strcat (cp, "	");
-	      strcat (cp, he->h_name);
+	      cp = stpcpy (cp, "\n");
+	      cp = stpcpy (cp, inet_ntoa (*(struct in_addr *)
+					  he->h_addr_list[j]));
+	      *cp++ = '\t';
+	      cp = stpcpy (cp, he->h_name);
 
 	      for (i = 0; he->h_aliases[i]; i++)
 		{
-		  strcat (cp, " ");
-		  strcat (cp, he->h_aliases[i]);
+		  cp = stpcpy (cp, " ");
+		  cp = stpcpy (cp, he->h_aliases[i]);
 		}
 	    }
-	  strcat (cp, "\n");
-	  len = strlen (cp);
+	  *cp++ = '\n';
+	  *cp = '\0';
 
-	  result.val.valdat_len = len;
-	  result.val.valdat_val = cp;
+	  result.val.valdat_len = strlen (result.val.valdat_val);
 	  result.stat = YP_TRUE;
 	}
       else
@@ -526,6 +546,8 @@ ypproc_first_2_svc (ypreq_nokey *key, struct svc_req *rqstp)
       else
         {
 	  datum dkey = ypdb_firstkey (dbp);
+
+	  /* XXX Replace strncmp */
 	  while (dkey.dptr != NULL && strncmp (dkey.dptr, "YP_", 3) == 0)
 	    {
 #if defined(HAVE_NDBM) /* This is much more faster then ypdb_nextkey, but
@@ -734,6 +756,9 @@ ypproc_xfr_2_svc (ypreq_xfr *xfr, struct svc_req *rqstp)
     {
       if (debug_flag)
 	yp_msg ("\t-> Ignored (not a valid source host)\n");
+      else
+	yp_msg ("ypproc_xfr: refuse to transfer map from %s",
+		inet_ntoa (rqhost->sin_addr));
 
       result.xfrstat = YPXFR_REFUSED;
       return &result;
@@ -743,7 +768,9 @@ ypproc_xfr_2_svc (ypreq_xfr *xfr, struct svc_req *rqstp)
     {
       if (debug_flag)
 	yp_msg ("\t-> Ignored (not a valid domain)\n");
-
+      else
+	yp_msg ("ypproc_xfr: refuse to transfer map from %s, no valid domain",
+		inet_ntoa (rqhost->sin_addr));
       result.xfrstat = YPXFR_NODOM;
       return &result;
     }
@@ -752,7 +779,9 @@ ypproc_xfr_2_svc (ypreq_xfr *xfr, struct svc_req *rqstp)
     {
       if (debug_flag)
 	yp_msg ("\t-> Ignored (map contains \"/\"!)\n");
-
+      else
+	yp_msg ("ypproc_xfr: refuse to transfer map from %s, no valid mapname",
+		inet_ntoa (rqhost->sin_addr));
       result.xfrstat = YPXFR_REFUSED;
       return &result;
     }
@@ -763,6 +792,9 @@ ypproc_xfr_2_svc (ypreq_xfr *xfr, struct svc_req *rqstp)
 	{
 	  if (debug_flag)
 	    yp_msg ("\t-> Ignored (no reserved port!)\n");
+	  else
+	    yp_msg ("ypproc_xfr: refuse to transfer %s from %s, no valid port",
+		    xfr->map_parms.map, inet_ntoa (rqhost->sin_addr));
 
 	  result.xfrstat = YPXFR_REFUSED;
 	  return &result;
@@ -770,7 +802,7 @@ ypproc_xfr_2_svc (ypreq_xfr *xfr, struct svc_req *rqstp)
     }
 
   /* If we have the map, check, if the master name is the same as in
-     the ypreq_xfr struct If we doesn't have the map, trust the host */
+     the ypreq_xfr struct. If we doesn't have the map, refuse. */
   dbp = ypdb_open(xfr->map_parms.domain, xfr->map_parms.map);
   if (dbp != NULL)
     {
@@ -788,6 +820,9 @@ ypproc_xfr_2_svc (ypreq_xfr *xfr, struct svc_req *rqstp)
 	      if (debug_flag)
 		yp_msg ("\t->Ignored (%s is not the master!)\n",
 			xfr->map_parms.peer);
+	      else
+		yp_msg ("ypproc_xfr: refuse to transfer %s from %s, not master",
+			xfr->map_parms.map, inet_ntoa (rqhost->sin_addr));
 
 	      result.xfrstat = YPXFR_NODOM;
 	      return &result;
@@ -805,14 +840,18 @@ ypproc_xfr_2_svc (ypreq_xfr *xfr, struct svc_req *rqstp)
 	}
       ypdb_close(dbp);
     }
-  /* If you wish to refuse the transfer of new maps, change the next
-     #if 0 statement to #if 1 */
-#if 0
+  /* If you wish to allow the transfer of new maps, change the next
+     #if 1 statement to #if 0 */
+#if 1
   else
     {
       /* We doesn't have the map, refuse the transfer */
       if (debug_flag)
 	yp_msg ("\t->Ignored (I don't have this map)\n");
+      else
+	yp_msg ("ypproc_xfr: refuse to transfer %s from %s, map doesn't exist",
+		xfr->map_parms.map, inet_ntoa (rqhost->sin_addr));
+
       result.xfrstat = YPXFR_REFUSED;
       return &result;
     }
@@ -830,8 +869,8 @@ ypproc_xfr_2_svc (ypreq_xfr *xfr, struct svc_req *rqstp)
 	sprintf (g, "%u", xfr->prog);
 	sprintf (p, "%u", xfr->port);
 	if (debug_flag)
-	  execl (ypxfr_command, "ypxfr", "--debug", "-d", xfr->map_parms.domain,
-		 "-h", xfr->map_parms.peer, "-C", t, g,
+	  execl (ypxfr_command, "ypxfr", "--debug", "-d",
+		 xfr->map_parms.domain, "-h", xfr->map_parms.peer, "-C", t, g,
 		 inet_ntoa (rqhost->sin_addr), p, xfr->map_parms.map, NULL);
 	else
 	  execl (ypxfr_command, "ypxfr", "-d", xfr->map_parms.domain, "-h",
@@ -842,7 +881,7 @@ ypproc_xfr_2_svc (ypreq_xfr *xfr, struct svc_req *rqstp)
 	exit (0);
       }
     case -1:
-      yp_msg ("fork(): %s", strerror (errno));
+      yp_msg ("Cannot fork: %s", strerror (errno));
       result.xfrstat = YPXFR_XFRERR;
     default:
       result.xfrstat = YPXFR_SUCC;
@@ -855,7 +894,8 @@ ypproc_xfr_2_svc (ypreq_xfr *xfr, struct svc_req *rqstp)
 
 /* Close all cached filehandles */
 void *
-ypproc_clear_2_svc (void *dummy, struct svc_req *rqstp)
+ypproc_clear_2_svc (void *dummy __attribute__ ((unused)),
+		    struct svc_req *rqstp)
 {
   static int *result;
   static int rval;
@@ -914,6 +954,7 @@ ypall_encode (ypresp_key_val *val, void *data)
 #if defined(HAVE_NDBM)
 
   dkey = dbm_nextkey (((ypall_data_t) data)->dbm);
+  /* XXX Replace strncmp */
   while (dkey.dptr != NULL && strncmp (dkey.dptr, "YP_", 3) == 0)
     dkey = dbm_nextkey (((ypall_data_t) data)->dbm);
 
@@ -924,6 +965,8 @@ ypall_encode (ypresp_key_val *val, void *data)
   oldkey.dptr = strndup (val->key.keydat_val, oldkey.dsize);
 
   dkey = ypdb_nextkey (((ypall_data_t) data)->dbm, oldkey);
+
+  /* XXX Replace strncmp */
   while (dkey.dptr != NULL && strncmp (dkey.dptr, "YP_", 3) == 0)
     {
       free (oldkey.dptr);
@@ -957,11 +1000,12 @@ ypall_encode (ypresp_key_val *val, void *data)
   return val->stat;
 }
 
+extern xdr_ypall_cb_t xdr_ypall_cb;
+
 ypresp_all *
 ypproc_all_2_svc (ypreq_nokey *nokey, struct svc_req *rqstp)
 {
   static ypresp_all result;
-  extern xdr_ypall_cb_t xdr_ypall_cb;
   int valid;
   int mangle = 0;
 
@@ -998,13 +1042,32 @@ ypproc_all_2_svc (ypreq_nokey *nokey, struct svc_req *rqstp)
   if (valid > 200)
     mangle = valid - 200;
 
-  if (children < MAX_CHILDREN && fork ())
+  if (children < MAX_CHILDREN)
     {
-      children++;
-      return NULL;
+      switch (fork ())
+	{
+	case 0:
+	  ++forked;
+#ifdef DEBUG
+	  yp_msg ("ypserv has forked for ypproc_all(): pid=%i\n", getpid ());
+	  if (!forked)
+	    abort ();
+#endif
+	  /* Close all databases ! */
+	  ypdb_close_all ();
+	  break;
+	case -1:
+	  yp_msg ("Cannot fork: %s\n", strerror (errno));
+	  break;
+	default:
+	  ++children;
+	  return NULL;
+	  break;
+	}
     }
   else
-    forked++;
+    yp_msg ("WARNING: ypserv has to many running children ! We don't fork\n");
+
 
   xdr_ypall_cb.u.encode = NULL;
   xdr_ypall_cb.u.close = NULL;
@@ -1032,6 +1095,8 @@ ypproc_all_2_svc (ypreq_nokey *nokey, struct svc_req *rqstp)
       else
 	{
           datum dkey = ypdb_firstkey (data->dbm);
+
+	  /* XXX Replace strncmp */
 	  while (dkey.dptr != NULL && strncmp (dkey.dptr, "YP_", 3) == 0)
 	    {
 	      datum tkey = dkey;
@@ -1107,7 +1172,7 @@ ypproc_master_2_svc (ypreq_nokey * nokey, struct svc_req * rqstp)
       result.peer = NULL;
     }
 
-  if ((valid = is_valid_host (rqstp, nokey->map, nokey->domain)) < 1)
+  if ((valid = is_valid_host (rqstp, NULL, NULL)) < 1)
     {
       if (valid == 0)
 	{
@@ -1173,7 +1238,7 @@ ypproc_master_2_svc (ypreq_nokey * nokey, struct svc_req * rqstp)
 	      int i;
 
 	      /* put the eof string mark at the end of the string */
-	      for (i = 0; i < val.dsize; i++)
+	      for (i = 0; i < val.dsize; ++i)
 		hostbuf[i] = val.dptr[i];
 	      hostbuf[val.dsize] = '\0';
 	      ypdb_free (val.dptr);
@@ -1297,11 +1362,22 @@ add_maplist (ypmaplist ** mlhp, char *map)
   ypmaplist *mlp;
 #if defined(HAVE_NDBM)
 #if defined(sun) || defined(__sun__)
-  /* XXX We have real NDBM support. Need some more hacks here */
+  int len = strlen (map);
+
+  /* We have all maps twice: with .dir and with .pag. Ignore
+     .pag */
+  if (len > 3 && map[len - 4] == '.' && map[len - 3] == 'p' &&
+      map[len - 2] == 'a' && map[len - 1] == 'g')
+    return 0;
+
+  if (len > 3 && map[len - 4] == '.' && map[len - 3] == 'd' &&
+      map[len - 2] == 'i' && map[len - 1] == 'r')
+    map[len - 4] = '\0';
 #else
   int len = strlen (map);
 
-  if (map[len - 3] == '.' && map[len - 2] == 'd' && map[len - 1] == 'b')
+  if (len > 2 && map[len - 3] == '.' && map[len - 2] == 'd' &&
+      map[len - 1] == 'b')
     map[len - 3] = '\0';
 #endif
 #endif
