@@ -1,29 +1,21 @@
-/*
-** Copyright (c) 1996, 1997, 1998 Thorsten Kukuk
-**
-** This file is part of the NYS YP Server.
-**
-** The NYS YP Server is free software; you can redistribute it and/or
-** modify it under the terms of the GNU General Public License as
-** published by the Free Software Foundation; either version 2 of the
-** License, or (at your option) any later version.
-**
-** The NYS YP Server is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-** General Public License for more details.
-**
-** You should have received a copy of the GNU General Public
-** License along with the NYS YP Server; see the file COPYING.  If
-** not, write to the Free Software Foundation, Inc., 675 Mass Ave,
-** Cambridge, MA 02139, USA.
-**
-** Author: Thorsten Kukuk <kukuk@uni-paderborn.de>
-*/
+/* Copyright (c) 1996, 1997, 1998, 1999 Thorsten Kukuk
+   This file is part of the NYS YP Server.
+   Author: Thorsten Kukuk <kukuk@suse.de>
 
-#ifndef LINT
-static const char rcsid[] = "$Id: yppush.c,v 1.14 1998/03/06 20:01:11 kukuk Exp $";
-#endif
+   The NYS YP Server is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License as
+   published by the Free Software Foundation; either version 2 of the
+   License, or (at your option) any later version.
+
+   The NYS YP Server is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   General Public License for more details.
+
+   You should have received a copy of the GNU General Public
+   License along with the NYS YP Server; see the file COPYING.  If
+   not, write to the Free Software Foundation, Inc., 675 Mass Ave,
+   Cambridge, MA 02139, USA. */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -92,7 +84,7 @@ struct hostlist *hostliste = NULL;
 
 static char *DomainName = NULL;
 int verbose_flag = 0;
-int debug_flag = 1;
+int debug_flag = 1; /* Needed for yp_msg */
 static char local_hostname[MAXHOSTNAMELEN + 2];
 static char *current_map;
 static u_int CallbackProg = 0;
@@ -179,7 +171,8 @@ yppush_err_string (enum yppush_status status)
 }
 
 void *
-yppushproc_null_1_svc (void *req, struct svc_req *rqstp)
+yppushproc_null_1_svc (void *req __attribute__ ((unused)),
+		       struct svc_req *rqstp __attribute__ ((unused)))
 {
   static void *resp = NULL;
 
@@ -331,7 +324,7 @@ hostcmp (char *h1, char *h2)
   if (p && !s)
     *p = 0;
 
-  return strcmp (buf1, buf2);
+  return strcasecmp (buf1, buf2);
 }
 
 static char *
@@ -373,21 +366,23 @@ get_dbm_entry (char *key)
   dkey.dsize = strlen (dkey.dptr);
 #if defined(HAVE_LIBGDBM)
   dval = gdbm_fetch (dbm, dkey);
-  gdbm_close (dbm);
 #elif defined(HAVE_NDBM)
   dval = dbm_fetch (dbm, dkey);
-  dbm_close (dbm);
 #endif
   if (dval.dptr == NULL)
-    return NULL;
+    val = NULL;
   else
     {
       val = malloc (dval.dsize + 1);
       strncpy (val, dval.dptr, dval.dsize);
       val[dval.dsize] = 0;
-
-      return val;
     }
+#if defined(HAVE_LIBGDBM)
+  gdbm_close (dbm);
+#elif defined(HAVE_NDBM)
+  dbm_close (dbm);
+#endif
+  return val;
 }
 
 static u_int
@@ -423,18 +418,18 @@ getordernum (void)
 }
 
 /*
-   ** Create with the ypservers or slaves.hostname map a list with all
-   ** slave servers we should send the new map
- */
+** Create with the ypservers or slaves.hostname map a list with all
+** slave servers we should send the new map
+*/
 /* NetBSD has a different prototype in struct ypall_callback */
 #if defined(__NetBSD__)
 static int
 add_slave_server (u_long status, char *key, int keylen,
-		  char *val, int vallen, void *data)
+		  char *val, int vallen, void *data __attribute__ ((unused)))
 #else
 static int
-add_slave_server (int status, const char *key, int keylen,
-		  const char *val, int vallen, const char *data)
+add_slave_server (int status, char *key, int keylen,
+		  char *val, int vallen, char *data __attribute__ ((unused)))
 #endif
 {
   char host[YPMAXPEER + 2];
@@ -479,7 +474,7 @@ add_slave_server (int status, const char *key, int keylen,
 }
 
 static void
-child_sig_int (int sig)
+child_sig_int (int sig __attribute__ ((unused)))
 {
   if (CallbackProg != 0)
     svc_unregister (CallbackProg, 1);
@@ -492,8 +487,8 @@ yppush_foreach (const char *host)
   SVCXPRT *CallbackXprt;
   CLIENT *PushClient = NULL;
   struct ypreq_xfr req;
-  struct timeval tv =
-  {10, 0};
+  struct ypresp_xfr resp;
+  struct timeval tv = {10, 0};
   u_int transid;
   char server[YPMAXPEER + 2];
   int sock;
@@ -526,7 +521,7 @@ yppush_foreach (const char *host)
       exit (-1);
     }
 
-  PushClient = clnt_create (server, YPPROG, YPVERS, "tcp");
+  PushClient = clnt_create (server, YPPROG, YPVERS, "udp");
   if (PushClient == NULL)
     {
       yp_msg ("%s", host);
@@ -583,12 +578,15 @@ yppush_foreach (const char *host)
 	}
 
       if (clnt_call (PushClient, YPPROC_XFR, (xdrproc_t) xdr_ypreq_xfr,
-		     (void *) &req, (xdrproc_t) xdr_void, NULL, tv)
-	  != RPC_SUCCESS)
+		     (caddr_t) &req, (xdrproc_t) xdr_ypresp_xfr,
+		     (caddr_t) &resp, tv) != RPC_SUCCESS)
 	{
 	  clnt_perror (PushClient, "YPPUSH: Cannot call YPPROC_XFR");
 	  kill (transid, SIGTERM);
 	}
+
+      /* XXX Check resp.xfrstat here ! */
+
 #if HAVE_WAIT4
       wait4 (transid, &sock, 0, &r);
 #else
@@ -607,7 +605,7 @@ yppush_foreach (const char *host)
 }
 
 static void
-sig_child (int sig)
+sig_child (int sig __attribute__ ((unused)))
 {
   int status;
 
@@ -638,16 +636,16 @@ main (int argc, char **argv)
 
   struct ypall_callback f;
   enum ypstat y;
-  struct sigaction a;
+  struct sigaction sig;
 
-  a.sa_handler = sig_child;
-  sigemptyset (&a.sa_mask);
+  sig.sa_handler = sig_child;
+  sigemptyset (&sig.sa_mask);
 #if defined(linux) || (defined(sun) && defined(__srv4__))
-  a.sa_flags = SA_NOMASK;
+  sig.sa_flags = SA_NOMASK;
   /* Do  not  prevent  the  signal   from   being
      received from within its own signal handler. */
 #endif
-  sigaction (SIGCHLD, &a, NULL);
+  sigaction (SIGCHLD, &sig, NULL);
 
   while (1)
     {
@@ -685,7 +683,8 @@ main (int argc, char **argv)
 	  break;
 	case 'h':
 	  /* we can handle multiple hosts */
-	  if ((tmp = (struct hostlist *) malloc (sizeof (struct hostlist))) == NULL)
+	  tmp = (struct hostlist *) malloc (sizeof (struct hostlist));
+	  if (tmp == NULL)
 	    {
 	      yp_msg ("malloc() failed: %s", strerror (errno));
 	      return 1;
@@ -704,6 +703,7 @@ main (int argc, char **argv)
 	  Usage (1);
 	}
     }
+
   argc -= optind;
   argv += optind;
 
@@ -753,7 +753,7 @@ main (int argc, char **argv)
 #endif
       if (y && y != YP_NOMORE)
 	{
-	  yp_msg ("Could not read ypservers: %d %s\n",
+	  yp_msg ("Could not read ypservers map: %d %s\n",
 		  y, yperr_string (y));
 	}
     }
@@ -764,7 +764,7 @@ main (int argc, char **argv)
 
       current_map = *argv++;
       val = get_dbm_entry ("YP_MASTER_NAME");
-      if (val && strcmp (val, local_hostname) != 0)
+      if (val && strcasecmp (val, local_hostname) != 0)
 	{
 	  yp_msg ("YPPUSH: %s is not the master for %s, try it from %s.\n",
 		  local_hostname, current_map, val);
