@@ -1,4 +1,4 @@
-/* Copyright (c) 1996, 1997, 1998 Thorsten Kukuk
+/* Copyright (c) 1996, 1997, 1998, 1999 Thorsten Kukuk
 
    This file is part of the NYS YP Server.
 
@@ -17,11 +17,7 @@
    not, write to the Free Software Foundation, Inc., 675 Mass Ave,
    Cambridge, MA 02139, USA.
 
-   Author: Thorsten Kukuk <kukuk@uni-paderborn.de> */
-
-#ifndef LINT
-static const char rcsid[] = "$Id: ypserv_conf.c,v 1.9 1998/10/13 18:32:03 kukuk Exp $";
-#endif
+   Author: Thorsten Kukuk <kukuk@suse.de> */
 
 #if defined(HAVE_CONFIG_H)
 #include "config.h"
@@ -44,7 +40,7 @@ static const char rcsid[] = "$Id: ypserv_conf.c,v 1.9 1998/10/13 18:32:03 kukuk
 #include <compat/strdup.c>
 #endif
 
-int
+static int
 getipnr (char *n, char *network, char *netmask)
 {
   char *m;
